#include "sig.h"
#include "mqueue.h"



#define SWITCH_CHAN SIGRTMIN +1
#define CHANS SIGRTMIN +2
#define OUT1 1
#define OUT2 2
#define IN1 1
#define IN2 2
#define IN3 2


#define DEFAULT_NBYTES 1
int nbytes = DEFAULT_NBYTES;

#define MQ_IN1  "/mq_in1"
#define MQ_IN2  "/mq_in2"
#define MQ_IN3  "/mq_in3"
#define MQ_OUT1  "/mq_out1"
#define MQ_OUT2  "/mq_out2"
#define MQ_SWITCH  "/mq_switch"
int current_chan = OUT1;

mqd_t mq_in1, mq_in2, mq_in3; /* one queue for each input channel */
mqd_t mq_out1, mq_out2; /* one queue for each output */
mqd_t mq_switch; /* one queue for switch */
struct mq_attr ma; /* queue attributes */
int err;

char buf[1];

void change_chan(int signum, siginfo_t *data, void *extra) {
  if(current_chan == 1) current_chan = 2;
  else current_chan = 1; 
}

void input_chans(int signum, siginfo_t *data, void *extra) {
  unsigned int pri;
  if(data -> si_value.sival_int == IN1) {
    if((err = mq_receive(mq_in1, &buf[0], nbytes, &pri)) < 0) {
      exit(3);
    };
  }else { /* similarly for IN2 and IN3 */
  };
  if(current_chan == 1) {
    if((err = mq_send(mq_out1, &buf[0], nbytes, 0)) < 0) {
      exit(3);
    };
  }else {
    if((err = mq_send(mq_out1, &buf[0], nbytes, 0)) < 0) {
      exit(3);
    };
  }

}

int select() {

  sigset_t mask, omask;
  struct sigaction s, os;

  struct sigevent se;

  /* set up signal mask */
  sigemptyset(&mask);
  sigaddset(&mask, SWITCH_CHAN);
  sigaddset(&mask, CHANS);

  s.sa_flags = 0;
  s.sa_mask = mask;


  /* set up handlers */
  s.sa_sigaction = & change_chan;
  sigaction(SWITCH_CHAN, &s, &os);

  s.sa_sigaction = &input_chans;
  sigaction(CHANS, &s, &os);


  /* set message queues attributes*/
  ma.mq_flags = 0;    /* No special behavior */
  ma.mq_maxmsg = 1;
  ma.mq_msgsize = nbytes;

  if (( mq_in1 = mq_open(MQ_IN1, O_RDONLY, MODE, &ma)) < 0) {
        /* indicate error */
        exit(1);
  };
  /* similarly for the other two input queues */ 

 if (( mq_out1 = mq_open(MQ_OUT1, O_CREAT|O_EXCL, MODE, &ma)) < 0) {
        /* indicate error */
        exit(1);
  };
  /* similarly for the other output queues */
    


  if (( mq_switch = mq_open(MQ_SWITCH, O_RDONLY, MODE, &ma)) < 0) {
        /* indicate error */
        exit(1);
  };
   

  sigprocmask(SIG_BLOCK, &mask, &omask);

  se.sigev_notify = SIGNAL;
  se.sigev_signo = CHANS;
  se.sigev_value.sival_int = IN1;
  if((err = mq_notify(mq_in1, & se)) < 0) {
    exit(3);
  };

  /* similarly for other input channels */


  se.sigev_notify = SIGNAL;
  se.sigev_signo = SWITCH_CHAN;
  se.sigev_value.sival_int = 0;
  if((err = mq_notify(mq_switch, & se)) < 0) {
    exit(3);
  };
  
  sigprocmask(SIG_UNBLOCK, &mask, &omask);

  /* need to sleep here*/
}

